@@ -1,3 +1,7 @@
+
+#' @include aaa_generics.R
+NULL
+
 ####################################################################################
 ####			SaemixRes class - definition				####
 ####################################################################################
